# Copyright © 2022 Arrikto Inc.  All Rights Reserved.

"""Kale SDK.

This script uses an ML pipeline to create a model artifact ID for an XGBoost
model.
"""

import xgboost as xgb

from typing import Tuple

from sklearn.model_selection import train_test_split
from sklearn.datasets import fetch_california_housing

from kale.ml import Signature
from kale.types import MarshalData
from kale.sdk import pipeline, step
from kale.common import mlmdutils, artifacts


@step(name="data_loading")
def load_split_dataset() -> Tuple[MarshalData, MarshalData]:
    """Fetch California Housing dataset."""
    # get data and target of the dataset
    california_housing = fetch_california_housing()
    x = california_housing.data
    y = california_housing.target

    # split the dataset
    x_train, _, y_train, _ = train_test_split(x, y, test_size=.2,
                                              random_state=42)
    return x_train, y_train


@step(name="model_training")
def train(x: MarshalData, y: MarshalData) -> MarshalData:
    """Train a XGBRegressor model."""
    model = xgb.XGBRegressor(objective='reg:squarederror',
                             colsample_bytree=1,
                             eta=0.3,
                             learning_rate=0.1,
                             max_depth=5,
                             alpha=10,
                             n_estimators=2000)
    model.fit(x, y)
    return model


@step(name="register_model")
def register_model(model: MarshalData, x: MarshalData, y: MarshalData):
    mlmd = mlmdutils.get_mlmd_instance()

    signature = Signature(
        input_size=[1] + list(x[0].shape),
        output_size=[1] + list(y[0].shape),
        input_dtype=x.dtype,
        output_dtype=y.dtype)

    model_artifact = artifacts.XGBoostModel(
        model=model,
        description="A simple XGBRegressor model",
        version="1.0.0",
        author="Kale",
        signature=signature,
        tags={"app": "xgboost"}).submit_artifact()

    mlmd.link_artifact_as_output(model_artifact.id)
    return


@pipeline(name="regression", experiment="xgboost")
def ml_pipeline():
    """Run the ML pipeline."""
    x_train, y_train = load_split_dataset()
    model = train(x_train, y_train)
    register_model(model, x_train, y_train)


if __name__ == "__main__":
    ml_pipeline()
